/*
  Library created by Andrew Mascolo : 2/18/2014
  This library is an extension to Henning Karlsen's UTFT, UTouch, ITDB02_Graph16 and ITDB02_Touch.
  
  Function ideas from others:
    rounded_Square :: robtillaart
	drawMoon       :: robtillaart
	smiley_Face	   :: robtillaart
	HourGlass      :: robtillaart
	SpeechBubble   :: robtillaart
	drawGauge      :: cyclegadget 
  
*/

#ifndef TFT_Extension_h
#define TFT_Extension_h
#include <Arduino.h>

#include <UTFT.h>
#include <UTouch.h>

#include <math.h>

//=====================COLOR_PALLET==========================
#define BLACK   0,0,0
#define LIGHT_RED	255,160,160
#define RED     255,0,0
#define DARK_RED    180,0,0
#define LIGHT_ORANGE 255,234,150
#define ORANGE  210,135,0
#define YELLOW  255,255,0
#define GREEN   0,255,0
#define BLUE    0,0,255
#define LIGHT_BLUE 160,160,255
#define PURPLE  255,0,255
#define CYAN    0,255,255
#define GREY    128,128,128
#define WHITE   255,255,255
//==================END_OF_COLOR_PALLET======================

//=====================TOUCH_DEFINES=========================
#define PORTRAIT	0
#define LANDSCAPE	1
#define Big 1
#define Small 0
#define FILL 1
#define NOFILL 0
#define ROUNDED 1
#define NOTROUNDED 0

#define swap(type, A, B) {type T = A; A = B; B = T;}
#define deg_to_rad	0.01745 + 3.1415 // 0.01745 = degrees to radians, 1.57 = half PI 
#define Button_Groups 5
#define Num_Of_Buttons 10
//=================END_OF_TOUCH_DEFINES======================

//===================DISPLAY_DEFINES=========================
#define up 0
#define down 180
#define left 90
#define right 270
//==================END_OF_DISPLAY_DEFINES===================

//========================MOON_PHASES========================
#define WANING_GIBBOUS 7
#define THIRD_QUARTER 6
#define WANING_CRESCENT 5 
#define NEW 4
#define WAXING_CRESCENT 3
#define FIRST_QUARTER 2
#define WAXING_GIBBOUS 1
#define FULL 0 
//===========================================================

//=======================SMILEY_FACES========================
#define EVIL_PLAN 5,0
#define HAPPY 0,1
#define JOY 2,1
#define CONTENT 3,1
#define EVIL_PLAN2 5,1
#define DISAPOINTMENT 3,2
#define HATE 5,2
#define MONDAYS 9,2
#define OVER_JOY 0,3
#define OVER_JOY2 2,3
#define XD 8,3
#define EVIL_PLAN3 5,3
#define YAY 6,3
#define NO_THANKS  2,4
#define SAD 3,4
#define WHOA 7,4
#define DULL 9,4
#define GRUMPY_CAT 7,5
#define ASHAMED 3,5
#define HATE 5,5
#define NO 9,5
#define YEA 1,6
#define EXCITED 7,6
#define GAHH 1,7
#define GET_OUT 5,7
#define THE_HORROR 7,7
#define CRY_BABY 8,7
//========================================================

class TFT_Extension
{
	public:
	 
				TFT_Extension(UTFT *Disp, UTouch *Touch, bool Orient);  
		bool    TouchButton(int x1, int y1, int x2, int y2, int xc = -1, int yc = -1);
		bool    TouchButton_Draw(int x1, int y1, int x2, int y2, uint8_t buttonNumber, int xc = -1, int yc = -1);
		bool 	TextButton(char *str, bool font_size, int x1, int y1, int x2, int y2, uint8_t R, uint8_t G, uint8_t B);
		bool    LatchButton(int x1, int y1, int x2, int y2, uint8_t buttonNumber, int xc = -1, int yc = -1);
		bool    LatchButton_Draw(int x1, int y1, int x2, int y2, uint8_t buttonNumber, int xc = -1, int yc = -1);
		bool    TouchCircle(int cx, int cy, int radius, int tx = -1, int ty = -1);
        bool    TouchCircle_Draw(int cx, int cy, int radius, uint8_t circleNumber , int tx = -1, int ty = -1);
		bool    LatchCircle(int cx, int cy, int radius, uint8_t circleNumber, int tx = -1, int ty = -1);
		bool    LatchCircle_Draw(int cx, int cy, int radius, uint8_t circleNumber, int tx = -1, int ty = -1);
        bool    TouchTriangle(int x1, int y1, int base, int deg, int tx = -1, int ty = -1);
		bool	TouchTriangle_Draw(int x1, int y1, int base, int deg, uint8_t triangleNumber, int Tx = -1, int Ty = -1);
		bool	LatchTriangle(int x1, int y1, int base, int deg, uint8_t triangleNumber, int Tx = -1, int Ty = -1);
		bool	LatchTriangle_Draw(int x1, int y1, int base, int deg, uint8_t triangleNumber, int Tx = -1, int Ty = -1);
		bool    TouchDelayButton(int x1, int y1, int x2, int y2, unsigned long time);
		bool	TouchDelayCircle(int x1, int y1, int base, unsigned long time);
		bool	TouchDelayTriangle(int x1, int y1, int base, int deg, unsigned long time);
		int  	TotalRadioButtons(uint8_t *num, uint8_t group);
		bool 	RadioButton(int x1, int y1, int x2, int y2, uint8_t buttonNumber, uint8_t group);
		int  	TotalRadioCircleButtons(uint8_t *num, uint8_t group);
		bool 	RadioCircleButton(int cx, int cy, int radius, uint8_t buttonNumber, uint8_t group);
		void 	ResetRadioCircleButton(uint8_t group);
		void 	ResetRadioButton(uint8_t group);
		uint8_t	TouchArc(int cx, int cy, int radius, int thickness, int start, int stop);
		uint8_t	HorSlider(int x1, int y1, int x2, int y2, byte count, byte R, byte G, byte B);
		uint8_t	VertSlider(int x1, int y1, int x2, int y2, byte count, byte R, byte G, byte B);
		//====================END_OF_TOUCH=====================
		
		//======================DISPLAY========================
		void	DisplaySize(int X, int Y);
		void	SetTouchButtonColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill, bool rounded);
		void	SetTouchCircleColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill);
		void	SetTouchTriangleColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill);
		void	SetLatchButtonColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill, bool rounded);
		void	SetLatchCircleColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill);
		void	SetLatchTriangleColors(uint8_t ButtonNumber, uint8_t Rcolor1, uint8_t Gcolor1, uint8_t Bcolor1,uint8_t Rcolor2, uint8_t Gcolor2, uint8_t Bcolor2, bool fill);
		void	RB_Text(char * text, bool size, uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Text(char * text, bool size, uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void    RB_Outer_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void    RCB_Outer_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void 	RB_Toggled_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Toggled_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RB_Untoggled_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Untoggled_Color(uint8_t ButtonNumber, uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RB_Outer_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Outer_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RB_Toggled_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Toggled_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RB_Untoggled_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	RCB_Untoggled_Color_By_Group(uint8_t Group, uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RB_Outer_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RCB_Outer_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RB_Toggled_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RCB_Toggled_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RB_Untoggled_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	SetAll_RCB_Untoggled_Color( uint8_t Rcolor, uint8_t Gcolor, uint8_t Bcolor);
		void	Triangle(int x1, int y1,int x2,int y2,int x3,int y3);
        void 	drawTriangle(int x1,int y1,int base, int deg);
		void	fillTriangle(int x1,int y1,int base, int deg);
		void	Polygon(int cx, int cy, int sides, int diameter, byte R, byte G, byte B, bool fill, float deg = 0);
		void	fillPoly(int x1, int y1, int x2, int y2, int x3, int y3);
		void	drawStar(int cx, int cy, int diameter, byte R, byte G, byte B, bool fill, float factor = 2.0);
		void	drawArc(int cx, int cy, int radius, int thickness,int start, int stop, byte R, byte G, byte B);
		void	drawOvalArc(int cx, int cy, int Xradius, int Yradius, int Xthickness, int Ythickness, int start, int stop, byte R, byte G, byte B);
		void	drawMoon(int cx, int cy, int radius, uint8_t phase);
		void	rounded_Square(int cx,int cy,int h, int w, float radius, byte R, byte G, byte B, bool fill);
		void	smiley_Face(int cx, int cy, int radius, byte eyes, byte mouth);
		void	HourGlass(int cx, int cy, int height, int time);
		void	SpeechBubble(char * str, int cx, int cy, float radius, int deg);
		void	drawGauge(int cur, int pos_x, int pos_y, int start, int stop, int rad);
		//==================END_OF_DISPLAY=====================	
	
	protected:
		UTFT		*_Disp;
		UTouch		*_Touch; 		
		
	private:
		void	Setup();
		float   Area(int Ax, int Ay, int Bx, int By, int Cx, int Cy);		
        double 	Cx,Cx1,Cx2;
        double 	Cy,Cy1,Cy2;
		int		touchX, touchY;
		bool    _Orient;
		int		DispX, DispY;
		uint8_t TotalRB_Group;
		uint8_t TotalRCB_Group;	
		uint8_t percent;
		bool	locked;
		uint8_t flip;
		int		rad;
		unsigned long T;
		unsigned long B_current_time, C_current_time, T_current_time;
		bool	B_Touched, B_timeout;
		bool	C_Touched, C_timeout;
		bool	T_Touched, T_timeout;
		uint8_t Butlocked[Button_Groups], Cirlocked[Button_Groups];
		bool	ButLOCK, CirLOCK;
		int		B_lastx1, B_lasty1, B_lastx2, B_lasty2;
		int		C_lastcx, C_lastcy;
		double	x, oldx;
        double	y, oldy;		
};
#endif